using System;
using System.Collections.Generic;
using System.IO;
using DataDynamics.Reports.Extensibility.Rendering.IO;

namespace DDRonWeb
{
    public sealed class MemoryStreamProvider : StreamProvider
    {
        private int streamCount = 0;

        internal class MemoryStreamInfo : StreamInfo
        {
            private readonly MemoryStream stream;

            public MemoryStreamInfo(Uri uri, string mimeType) : base(uri, mimeType )
            {
                this.stream = new MemoryStream( 1024 );
            }

            public override Stream OpenStream()
            {
                return stream;
            }
        }

        private readonly List<MemoryStreamInfo> streams;

        public MemoryStreamProvider()
        {
            streams = new List<MemoryStreamInfo>(4);
        }

        public override StreamInfo CreatePrimaryStream( string mimeType, string suggestedFileExtension )
        {
            if( streams.Count > 0 )
                throw new Exception( "Primary stream already created" );

            Uri uri = new Uri( "file://report." + suggestedFileExtension );
            MemoryStreamInfo info = new MemoryStreamInfo( uri, mimeType );

            streams.Add( info );

            return info;
        }

        public override StreamInfo CreateSecondaryStream( string relativeName, string mimeType, string suggestedFileExtension )
        {
            if( streams.Count == 0 )
                throw new Exception( "Primary stream hasn't been created yet" );

            Uri uri = new Uri( "cid:" + relativeName + "." + suggestedFileExtension );
            MemoryStreamInfo info = new MemoryStreamInfo( uri, mimeType );

            streams.Add( info );

            return info;
        }

        public override StreamInfo GetPrimaryStream()
        {
            if( streams.Count == 0 )
                return null;

            return streams[ 0 ];
        }

        public override StreamInfo[] GetSecondaryStreams()
        {
            if( streams.Count > 1 )
            {
                MemoryStreamInfo[] secondaryStreams = new MemoryStreamInfo[streams.Count - 1];
                streams.CopyTo( 1, secondaryStreams, 0, streams.Count - 1 );

                return secondaryStreams;
            }

            return new MemoryStreamInfo[0];
        }
    }
}